import CaptchaService from "../services/captcha.js";
import ProxyService from "../services/proxy.js";
import config from '../config.js';
import fs from 'fs/promises';

class CaptchaController {
    async new(req, res) {
        try {
            const proxy = await ProxyService.take();
            const id = await CaptchaService.new(proxy);

            return res.status(200).send({"id": id});
        }  catch (e) {
            console.log(e)
            return res.status(500).send({"message": "Unknown server error"});
        }
    }

    async get(req, res) {
        try {
            const id = req.params.id;
        
            const hash = await CaptchaService.get(id);
            if (hash == undefined) {
                return res.status(404).send({"message": "captcha not found"});
            }
            const image = await fs.readFile(`${config.data_dir}/${hash}.jpeg`);

            return res.status(200).send(image)
        } catch (e) {
            console.log(e)
            return res.status(500).send({"message": "Unknown server error"});
        }
    }

    async solve (req, res) {
        try {
            const id = req.params.id;
            const solution = req.body["solution"];
            if (solution == undefined || solution.length != 6) {
                return res.status(400).send({"message": 'please, send a valid solution. Example: {"solution":"123456"}'});
            }
            
            if (!await CaptchaService.check_and_save_solution(id, solution))
                return res.status(409).send({"message": "Solution is not correct"});
            
            let proxy_id = await CaptchaService.get_assigned_proxy(id);
            await ProxyService.give_back(id);
            return res.status(200).send({"message": "Successful"});
        }  catch (e) {
            console.log(e)
            return res.status(500).send({"message": "Unknown server error"});
        }   
    }
}

export default new CaptchaController();